package SeMeet::Controller::Meets;
use Mojo::Base 'Mojolicious::Controller', -signatures;

use UUID::URandom qw(create_uuid_string);
use Mojo::JWT;

use constant AVATAR_SIZE          => 320;
use constant JITSI_TOKEN_LIFETIME => 3600 * 24;

sub create($c) {
    $c->openapi->valid_input or return;
    my $args = $c->req->json;

    return $c->error(403, 'Access denied') if ! $c->stash->{permissions}{create};

    my $exists = $c->schema->resultset('Meet')->count({
        deleted => undef,
        name    => $args->{name},
    });

    return $c->error(400, 'DUPLICTE_NAME') if $exists;

    my $guard  = $c->schema->txn_scope_guard;

        my $meet = $c->schema->resultset('Meet')->create({
            uuid     => create_uuid_string(),
            owner_id => $c->stash->{user}->id,
            name     => $args->{name},
        });

    $guard->commit;

    $c->trace(\'User %s create meet "%s" with id %d',
        $c->stash->{user}->username,
        $meet->name,
        $meet->id,
    );

    $c->render(
        status  => 201,
        openapi => { id => $meet->id },
    );
}

sub get($c) {
    $c->openapi->valid_input or return;
    my $args = $c->req->json;

    my $meet = $c->_get($c->stash->{id}, 0) // return;

    $c->render(openapi => $c->spec_filter({
        $meet->get_columns,
        groups => $meet->cached_groups,
        moderators  => $meet->cached_moderators,
        is_editable => ( $meet->owner_id == $c->stash->{user}->id),
    }, 'Meet'));
}

sub list($c) {
    $c->openapi->valid_input or return;
    my $args = $c->req->json;

    my $meets = $c->schema->resultset('Meet')->search({
        deleted  => undef,
        -or => [
            owner_id => $c->stash->{user}->id,
            "groups.group_id" => { '-in' => $c->stash->{groups} },
        ]
    },
    {
        order_by => 'name',
        join     => 'groups',
        distinct => 1,
    }
    );

    my @meets = ();

    MEET:
    while ( my $meet = $meets->next()) {
        push @meets, $c->spec_filter({
            $meet->get_columns,
            groups => $meet->cached_groups,
            moderators => $meet->cached_moderators,
            is_editable => ( $meet->owner_id == $c->stash->{user}->id),
        }, 'Meet')
    }

    $c->render(
        openapi => {
            count   => scalar( @meets ),
            records => \@meets,
        },
    );
}

sub add_groups($c) {
    $c->openapi->valid_input or return;
    my $args = $c->req->json;

    my $meet = $c->_get($c->stash->{id}, 1) // return;

    IDS:
    foreach my $id ( @{ $args->{groups} } ) {
        $meet->find_or_create_related('meet_groups',
            { group_id => $id }
        );
    }
    $meet->update_groups_cache();

    $c->render( status  => 204, text => '' );
}

sub add_moderators($c) {
    $c->openapi->valid_input or return;
    my $args = $c->req->json;

    my $meet = $c->_get($c->stash->{id}, 1) // return;

    IDS:
    foreach my $user ( @{ $args->{users} } ) {

        $meet->find_or_create_related('moderators',
            {
                octid => $user->{id},
                name  => $user->{name},
            }
        );
    }
    $meet->update_moderators_cache();

    $c->render( status  => 204, text => '' );
}

sub delete_group($c) {
    $c->openapi->valid_input or return;

    my $meet = $c->_get($c->stash->{id}, 1) // return;

    $meet->delete_related('meet_groups',
        { group_id => $c->stash->{group_id} }
    );
    $meet->update_groups_cache();

    $c->render( status  => 204, text => '' );
}

sub delete_moderator($c) {
    $c->openapi->valid_input or return;

    my $meet = $c->_get($c->stash->{id}, 1) // return;

    $meet->delete_related('moderators',
        { octid => $c->stash->{user_id} }
    );
    $meet->update_moderators_cache();

    $c->render( status  => 204, text => '' );
}

sub delete($c) {
    $c->openapi->valid_input or return;

    my $meet = $c->_get($c->stash->{id}, 1) // return;

    $meet->update({ deleted => \'now()'});

    $c->render( status  => 204, text => '' );
}

sub update($c) {
    $c->openapi->valid_input or return;
    my $args = $c->req->json;

    my $meet = $c->_get($c->stash->{id}, 1) // return;

    my $exists = $c->schema->resultset('Meet')->count({
        id      => { '!=' => $meet->id },
        deleted => undef,
        name    => $args->{name},
    });

    return $c->error(400, 'DUPLICTE_NAME') if $exists;

    $meet->update({
        name        => $args->{name},
        description => $args->{description},
    });

    $c->render( status  => 204, text => '' );
}

sub meet($c) { #NENI API!

    my $meet = $c->schema->resultset('Meet')->find({
        id => $c->stash->{id}
    });

    my $user = $c->schema->resultset('User')->find({
        id => $c->current_user->{id}
    });

    return $c->error(404, 'NOT_FOUND') if ! $meet;
    # CHECK MEET ACCESS

    $c->stash->{is_editable} = ( $meet->owner_id == $user->id);
    $c->stash->{token} = $user->meet_token($meet, $c->config);
    $c->stash->{meet}  = $meet;

    $c->render('meet');
}

sub _get($c, $id, $for_owner) {

    my $meet = $c->schema->resultset('Meet')->find({
        id => $id
    });

    return $c->error(404, 'NOT_FOUND') if ! $meet;

    if ( $for_owner ) {
        return $c->error(403, 'ACCESS_DENIED') if $meet->owner_id != $c->stash->{user}->id;
    }

    return $meet;
}

1;

